package com.rootsoft.audiorecorder;

import java.io.IOException;

import android.media.MediaRecorder;
import android.util.Log;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

@ShortName("AudioRecorder")
@Permissions(values = {"android.permission.WRITE_EXTERNAL_STORAGE", "android.permission.RECORD_AUDIO"})
@Author("XverhelstX")
@Version(2f)
public class AudioRecorder {
	
	private BA ba; 
	private boolean isRecording;
	
	//Audio Sources
	public int AS_CAMCORDER;
	public int AS_DEFAULT;
	public int AS_MIC;
	public int AS_VOICE_CALL;
	public int AS_VOICE_COMMUNICATION;
	public int AS_VOICE_DOWNLINK;
	public int AS_VOICE_RECOGNITION;
	public int AS_VOICE_UPLINK;
	
	public int OF_AMR_NB;
	public int OF_AMR_WB;
	public int OF_DEFAULT;
	public int OF_MPEG_4;
	public int OF_RAW_AMR;
	public int OF_THREE_GPP;
	
	public int AE_AAC;
	public int AE_AMR_NB;
	public int AE_AMR_WB;
	public int AE_DEFAULT;
	
	
	
	MediaRecorder mRecorder = new MediaRecorder();
	
	/**
     * Initializes AudioRecorder Library.
     *
     */
	
	public void Initialize(BA ba)
	{
	  this.ba = ba;
	  isRecording = false;
	  
	  //Audio Sources
	  AS_CAMCORDER = MediaRecorder.AudioSource.CAMCORDER;
	  AS_DEFAULT = MediaRecorder.AudioSource.DEFAULT;
	  AS_MIC = MediaRecorder.AudioSource.MIC;
	  AS_VOICE_CALL = MediaRecorder.AudioSource.VOICE_CALL;
	  AS_VOICE_COMMUNICATION = MediaRecorder.AudioSource.VOICE_COMMUNICATION;
	  AS_VOICE_DOWNLINK = MediaRecorder.AudioSource.VOICE_DOWNLINK;
	  AS_VOICE_RECOGNITION = MediaRecorder.AudioSource.VOICE_RECOGNITION;
	  AS_VOICE_UPLINK = MediaRecorder.AudioSource.VOICE_UPLINK;
	  
	  //Output Formats
	  OF_AMR_NB = MediaRecorder.OutputFormat.AMR_NB;
	  OF_AMR_WB = MediaRecorder.OutputFormat.AMR_WB;
	  OF_DEFAULT = MediaRecorder.OutputFormat.DEFAULT;
	  OF_MPEG_4 = MediaRecorder.OutputFormat.MPEG_4;
	  OF_RAW_AMR = MediaRecorder.OutputFormat.RAW_AMR;
	  OF_THREE_GPP = MediaRecorder.OutputFormat.THREE_GPP;
	  
	  //Audio Encoder
	  AE_AAC = MediaRecorder.AudioEncoder.AAC;
	  AE_AMR_NB = MediaRecorder.AudioEncoder.AMR_NB;
	  AE_AMR_WB = MediaRecorder.AudioEncoder.AMR_WB;
	  AE_DEFAULT = MediaRecorder.AudioEncoder.DEFAULT;
	  
	}
	
	/**
	 * Returns a boolean with true if the recorder is currently recording.
	 * @return 
	 */
	public boolean isRecording() {
		return isRecording;
	}
	
	/**
     * Gets the maximum value for audio sources.
     * 
     */
    @SuppressWarnings("static-access")
	public int getAudioSourceMax () {
        return mRecorder.getAudioSourceMax();
    }
    
    /**
     * Gets the maximum amplitude.
     * 
     */
    public int getAudioMaxAmplitude() {
        return mRecorder.getMaxAmplitude();
    }
    
    /**
     * Sets the number of audio channels for recording. 
     * Call this method before prepare(). 
     * Prepare() may perform additional checks on the parameter to make sure whether the specified number of audio channels are applicable.
     * API Level 8
     * @param numChannels
     */
    public void setAudioChannels (int numChannels) {
    	mRecorder.setAudioChannels(numChannels);
    }
    
    /**
     * Sets the audio encoder to be used for recording. 
     * If this method is not called, the output file will not contain an audio track. 
     * Call this after setOutputFormat() but before prepare().
     * 
     * @param audio_encoder the audio encoder to use.
     */
    public void setAudioEncoder (int audio_encoder) throws IllegalStateException {
    	mRecorder.setAudioEncoder(audio_encoder);
    }
    
    /**
     * Sets the audio encoding bit rate for recording. 
     * API Level 8
     * Call this method before prepare(). 
     * Prepare() may perform additional checks on the parameter to make sure whether the specified bit rate is applicable, 
     * and sometimes the passed bitRate will be clipped internally to ensure the audio recording can proceed smoothly based 
     * on the capabilities of the platform.
     * 
     * @param bitRate the audio encoding bit rate in bits per second.
     */
    public void setAudioEncodingBitRate (int bitRate)  {
    	mRecorder.setAudioEncodingBitRate(bitRate);
    }
    
    /**
     * Sets the audio sampling rate for recording. 
     * Call this method before prepare(). 
     * API Level 8
     * Prepare() may perform additional checks on the parameter to make sure whether the specified audio sampling rate is applicable. 
     * The sampling rate really depends on the format for the audio recording, as well as the capabilities of the platform. 
     * For instance, the sampling rate supported by AAC audio coding standard ranges from 8 to 96 kHz, the sampling 
     * rate supported by AMRNB is 8kHz, and the sampling rate supported by AMRWB is 16kHz. Please consult with the related audio coding 
     * standard for the supported audio sampling rate.
     * 
     * @param samplingRate the sampling rate for audio in samples per second.
     */
    public void setAudioSamplingRate (int samplingRate) {
    	mRecorder.setAudioSamplingRate(samplingRate);
    }
    
    /**
     * Sets the audio source to be used for recording. 
     * If this method is not called, the output file will not contain an audio track. 
     * The source needs to be specified before setting recording-parameters or encoders. 
     * Call this only before setOutputFormat().
     * 
     * @param audio_source the audio source to use
     */
    public void setAudioSource (int audio_source) throws IllegalStateException{
    	Log.i("B4A","Audio Source: " + audio_source);
    	mRecorder.setAudioSource(audio_source);
    }
    
    /**
     * Sets the maximum duration (in ms) of the recording session. 
     * Call this after setOutFormat() but before prepare(). 
     * 
     * @param ms the maximum duration in ms (if zero or negative, disables the duration limit)
     */
    public void setMaxDuration (int ms) throws IllegalArgumentException	 {
    	mRecorder.setMaxDuration(ms);
    }
    
    /**
     * Sets the maximum filesize (in bytes) of the recording session. 
     * Call this after setOutFormat() but before prepare().
     * 
     * @param bytes the maximum filesize in bytes (if zero or negative, disables the limit)
     */
    public void setMaxFileSize (long bytes) {
    	mRecorder.setMaxFileSize(bytes);
    }
    
    /**
     * Sets the path of the output file to be produced. 
     * Call this after setOutputFormat() but before prepare().
     * 
     * @param Dir The pathname to the directory
     * @param Filename The filename you wish to save for.
     */
    public void setOutputFile (String Dir, String Filename) throws IllegalStateException{
    	mRecorder.setOutputFile(Dir + "/" + Filename);
    }
    
    /**
     * Sets the format of the output file produced during recording. 
     * Call this after setAudioSource()/setVideoSource() but before prepare().
     * It is recommended to always use 3GP format when using the H.263 video encoder and AMR audio encoder. 
     * Using an MPEG-4 container format may confuse some desktop players.
     * 
     * @param OutputFormat the output format to use. The output format needs to be specified before setting recording-parameters or encoders.
     */
    public void setOutputFormat (int OutputFormat) throws IllegalStateException	{
    	mRecorder.setOutputFormat(OutputFormat);
    }
    
    /**
     * Begins capturing and encoding data to the file specified with setOutputFile(). 
     * Call this after prepare().
     */
    public void start () throws IllegalStateException{
    	isRecording = true;
    	mRecorder.start();
    }
    
    /**
     * Stops recording. 
     * Call this after start(). 
     * Once recording is stopped, you will have to configure it again as if it has just been constructed.
     */
    public void stop () {
    	isRecording = false;
    	mRecorder.stop();
    }
    
    /**
     * Prepares the recorder to begin capturing and encoding data. 
     * This method must be called after setting up the desired audio and video sources, encoders, file format, etc., but before start().
     * @throws IllegalStateException
     * @throws IOException
     */
    public void prepare () throws IllegalStateException, IOException {
    	mRecorder.prepare();
    }
    

}
